package it.enea.cruise.model.user;

import com.fasterxml.jackson.annotation.*;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;

import jakarta.persistence.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 
 * @author Gianluca D'Agosta Questa classe gestisce la tabella "Communities" del
 *         database con i dati di consumo delle Comunita' energetiche. Essa
 *         permette, con una relazione 1 a molti, di collegare tutti + gli
 *         utenti che parteticpano alla comunita'.
 *
 */

@Entity
@Getter
@Setter
@Table(name = "communities")
@JsonIdentityInfo(
		  generator = ObjectIdGenerators.PropertyGenerator.class, 
		  property = "communityid",
		  scope = User.class)
public class Community {
	/*
	 * Properties of the Community
	 */

	@Id
	@GeneratedValue
	@Column(name = "communityid")
	private Long id;

	@JsonInclude(JsonInclude.Include.NON_NULL)
	@Column(name = "communityname")
	private String communityName;

	@JsonInclude(JsonInclude.Include.NON_NULL)
	private String description;

	@JsonIgnore
	private String georeferenceformat;

	@JsonProperty("georeferencex")
	private Double georeferencex;

	@JsonProperty("georeferencey")
	private Double georeferencey;

	@JsonProperty("georeferencez")
	private Double georeferencez;
	
	/*
	 * Questo crea il legame con la tabella users_communities. Essendo
	 * uni-direzionale con questa tabella, la relazione e' uno (community) a molti
	 * (users) perche', dal lato della Comunita', essa puo' essere associata a
	 * molteplici utenti.
	 */
	@OneToMany(mappedBy = "community", cascade = CascadeType.ALL, orphanRemoval = true, fetch=FetchType.EAGER)
	@Fetch(value = FetchMode.SUBSELECT)
	//@JsonManagedReference(value = "community-communities")
	@JsonIgnoreProperties({"user", "community"})
	private List<UserCommunity> userCommunities = new ArrayList<>();


	/*
	 * Generic Methods
	 */
	public Community() {
	}

	public Community(String name) {
		this.communityName = name;
	}

	public Community(Long id, String name, String description) {
		this.id = id;
		this.communityName = name;
		this.description = description;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o)
			return true;
		if (o == null || getClass() != o.getClass())
			return false;
		Community u = (Community) o;
		return Objects.equals(communityName, u.communityName);
	}

	@Override
	public int hashCode() {
		return Objects.hash(communityName);
	}
}
