package it.enea.cruise.model.user;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;

import jakarta.persistence.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Getter
@Setter
@Entity
@Table(name = "parametricquery", schema = "cruise")
@JsonInclude(JsonInclude.Include.NON_NULL)
public class ParametricQuery {

    @Id
    @Column(name = "queryid")
    @JsonProperty("queryid")
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;
    private String sql;
    private String description;
    @Column(name = "querytype")
    private String queryType; // {SQL, API};

    @Transient
    @JsonProperty("metadata")
    @JsonInclude(JsonInclude.Include.NON_NULL)
    List<HashMap<String, String>> parameterMetadata;

    @ManyToMany
    @JoinTable(
			schema = "cruise",
            name = "queries_parameters",
            joinColumns = @JoinColumn
                    (name = "queryid"),
            inverseJoinColumns = @JoinColumn(name = "paramid"))
    @Fetch(FetchMode.SUBSELECT)
    @JsonProperty("parameters")
    @JsonIgnoreProperties("parametricquery")
    private List<LocalQueryParameter> parameters;

    @ManyToMany
    @JoinTable(
			schema = "cruise",
            name = "queries_admittedqueryvalues",
            joinColumns = @JoinColumn(name = "queryid"),
            inverseJoinColumns = @JoinColumn(name = "admittedqueryvalueid"))
    private List<AdmittedQueryValues> admittedqueryvalues;

    @Transient
    Map<String, List<String>> values = new HashMap<>();

    @PostLoad
    public void postLoad() {
        for (AdmittedQueryValues aqv : admittedqueryvalues) {
            List<String> tempList;
            if (values.containsKey(aqv.getType())) {
                tempList = values.get(aqv.getType());
                tempList.add(aqv.getValue());

            } else {
                tempList = new ArrayList<String>();
                tempList.add(aqv.getValue());
            }
            values.put(aqv.getType(), tempList);
        }
    }
}
