/*
package it.enea.cruise.service.es;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.BulkRequest;
import co.elastic.clients.elasticsearch.core.BulkResponse;
import co.elastic.clients.elasticsearch.core.IndexRequest;
import co.elastic.clients.elasticsearch.core.IndexResponse;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Service;

import java.sql.ResultSetMetaData;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
@RequiredArgsConstructor
public class DataIndexingService {
    private static final int BATCH_SIZE = 1000;
    private final ElasticsearchClient esClient;
    private final JdbcTemplate jdbcTemplate;


    public void indexDatabaseTable(String documentName, String query, String ...keys) {
        jdbcTemplate.setFetchSize(0); // Enable streaming mode
        //BulkRequest bulkRequest = new BulkRequest.Builder().build();
        jdbcTemplate.query(query, (ResultSetExtractor<Void>) rs -> {
            List<BulkOperation> bulkOperations = new ArrayList<>();
            int rowCount = 0;

            while (rs.next()) {
                Map<String, Object> rowMap = new HashMap<>();
                ResultSetMetaData metaData = rs.getMetaData();
                int columnCount = metaData.getColumnCount();

                for (int i = 1; i <= columnCount; i++) {
                    rowMap.put(metaData.getColumnName(i), rs.getObject(i));
                }

                bulkOperations.add(new BulkOperation.Builder()
                        .index(idx -> idx
                                .index(documentName)
                                .document(rowMap)
                        ).build()
                );

                rowCount++;

                // Execute bulk request when batch size is reached
                if (rowCount % BATCH_SIZE == 0) {
                    executeBulkRequest(bulkOperations);
                    bulkOperations.clear(); // Reset the bulk operations list
                }
            }

            // Final bulk request for remaining records
            if (!bulkOperations.isEmpty()) {
                executeBulkRequest(bulkOperations);
            }

            return null; // Required by the ResultSetExtractor interface
        });

for (Map<String, Object> row : rows) {
            StringBuilder keyBuilder = new StringBuilder();

            IntStream.range(0, keys.length)
                    .forEach(index -> {
                        String key = keys[index];
                        keyBuilder.append(key).append(index < keys.length - 1 ? "_" : "");
                    });

            indexRow(documentName, row, keyBuilder.toString());
        }

    }

    private void executeBulkRequest(List<BulkOperation> bulkOperations) {
        try {
            BulkRequest bulkRequest = new BulkRequest.Builder()
                    .operations(bulkOperations)
                    .build();

            BulkResponse bulkResponse = esClient.bulk(bulkRequest);

            if (bulkResponse.errors()) {
                // Handle failures here (e.g., log errors or retry)
                bulkResponse.items().forEach(item -> {
                    if (item.error() != null) {
                        System.out.println("Bulk insert error: " + item.error().reason());
                    }
                });
            } else {
                System.out.println("Batch insert succeeded for " + bulkOperations.size() + " records.");
            }
        } catch (Exception e) {
            e.printStackTrace(); // Handle exception (log or retry)
        }
    }

    private void indexRow(String documentName, Map<String, Object> row, String documentId) {
        try {

            // Construct the document as a Map
            Map<String, Object> document = new HashMap<>(row);

            // Index the document in Elasticsearch
            IndexRequest<Map<String, Object>> request = IndexRequest.of(i -> i
                    .index(documentName)
                    .id(documentId)
                    .document(document)
            );

            IndexResponse response = esClient.index(request);
            System.out.println("Indexed document ID: " + response.id());

        } catch (Exception e) {
            log.error("indexRow", e);
        }
    }
}
*/
