import {Component, OnDestroy, OnInit} from '@angular/core';
import {Router} from '@angular/router';
import {TranslateService} from '@ngx-translate/core';
import {Page} from "src/app/models/Page";
import {UserService} from 'src/app/services/user.service';
import {PageService} from 'src/app/services/page.service';
import {Subscription} from 'rxjs';

import {MenuItem} from 'primeng/api';
import {LanguageService} from "../../services/language.service";
import {AuthService} from "../../services/auth.service";

@Component({
    selector: 'app-header',
    templateUrl: './header.component.html',
    styleUrls: ['./header.component.css']
})
export class HeaderComponent implements OnInit, OnDestroy {
    //@ViewChild("menuBar") menuBar?: Menubar;

    title = 'lec';

    items: MenuItem[] = [];

    changingLanguage: string = "";

    languages: string[] = this.translate.getLangs();

    langItems: MenuItem[] = this.createMenuItemsFromLanguages(this.languages);

    pages: Page[] = [];
    alive: boolean = true;

    disposeOnDestroySubscriptions: Subscription[] = [];
    lastLoadedLang: string | null = null;

    constructor(
        public translate: TranslateService,
        private router: Router,
        public userService: UserService,
        private authService: AuthService,
        private languageService: LanguageService,
        private pageService: PageService
    ) {
    }

    ngOnDestroy() {
        this.alive = false;
        for (const subscription of this.disposeOnDestroySubscriptions)
            subscription.unsubscribe();
    }

    ngOnInit(): void {
        this.initMinimalMenu();
        this.disposeOnDestroySubscriptions.push(
            /*this.authService.onAuthorizationDone()
                .pipe(takeWhile(() => this.alive))
                .subscribe((authorization: Authorization) => {
                    if (this.userService.user)
                        this.loadPages();
                    this.lastLoadedLang = this.languageService.getLanguage();
                }),

            this.translate.onLangChange.subscribe(lang => {
                    const langHasChanged = lang.lang !== this.lastLoadedLang;

                    if (this.userService.user && langHasChanged) {
                        this.loadPages();
                    }
                    this.initMinimalMenu();
                    this.lastLoadedLang = lang.lang;
                }
            )*/
        );
    }

    loadPages() {
        this.pageService.getUserPages().subscribe(pages => {
            this.pages = pages;
            this.initMenu();
            this.translateMenuItems();
        });
    }

    initMinimalMenu() {
        if (this.userService.user) {
            this.items = [{
                icon: 'pi pi-flag',
                styleClass: 'menuLevel1',
                style: {'margin-left': 'auto'},
                items: this.langItems
            },
                {
                    label: 'LOGIN.SIGN_OUT',
                    styleClass: 'menuLevel1',
                    style: {'margin-left': 'auto'},
                    icon: 'my-margin-left pi pi-fw pi-sign-out',
                    command: () => this.logout(),
                    routerLink: '/'
                }

            ];
        } else {
            this.items = [{
                icon: 'pi pi-flag',
                styleClass: 'menuLevel1',
                items: this.langItems
            },
                {
                    label: 'LOGIN.SIGN_IN',
                    styleClass: 'menuLevel1',
                    style: {'margin-left': 'auto'},
                    icon: 'my-margin-left pi pi-fw pi-sign-in',
                    routerLink: '/login'
                }

            ];
        }
        this.translateMenuItems();
    }

    initMenu() {
        this.initMinimalMenu();

        if (!this.pages || this.pages.length === 0)
            return;

        this.items = [...this.items, ...this.createMenuItemsFromPages(this.pages)];
    }

    private createMenuItemsFromPages(pages: Page[]) {

        return pages.map(page => <MenuItem>{
            id: page.slug,
            label: page.name,
            routerLink: page.path,
            styleClass: "menuLevel1",
            state: {defaultPage: page.defaultPage},
            command: (event) => {
                const slug = event?.item?.id as string | undefined;
                if (slug)
                    this.pageService.getPageConfiguration(slug);
            }
        });
    }


    private createMenuItemsFromLanguages(languages: string[]) {

        return languages.map(language => <MenuItem>{
            label: language,
            styleClass: "menuLevel1",
            command: () => this.setActualLanguage(language)
        });
    }

    logout() {
        this.pages = [];
        this.initMenu();
        this.router.resetConfig(this.router.config.filter(o => !o.data || !o.data.dynamic));
        this.authService.removeAuthorization();

        /*this.disposeOnDestroySubscriptions.push(this.userService.logout().subscribe( {
            next: _ => {
                console.info("Logging out...")
                this.router.navigateByUrl("/");
            },
            error: err => {
                console.error("Error logging out: ", err);
                this.router.navigateByUrl("/");
            }
        }));*/
    }


    setActualLanguage(lang: string) {
        this.languageService.setLanguage(lang);
        this.changingLanguage = lang;
        this.translateMenuItems;
    }

    translateMenuItems() {

        this.items.forEach((menuItem: MenuItem) => {
            if (menuItem.label) {
                this.translate.get(menuItem.label || "").subscribe((translation: string) => {
                    menuItem.label = translation;
                });
            }
        });
    }
}
