import {Component, OnDestroy, OnInit} from '@angular/core';
import {environment} from "../../../environments/environment";
import {Subscription} from "rxjs";
import {UserService} from "../../services/user.service";
import {OAuthEvent, OAuthService} from "angular-oauth2-oidc";
import {AuthService} from "../../services/auth.service";

@Component({
  selector: 'app-logout-oid',
  templateUrl: './logout-oid.component.html',
  styleUrls: ['./logout-oid.component.css']
})
export class LogoutOidComponent implements OnInit, OnDestroy {
  provider: string = environment.oidProvider;
  oauthHandler?: Subscription;

  constructor(private userService: UserService, private authService: OAuthService, private appAuth: AuthService) {}

  ngOnInit(): void {
    // IMPORTANT: Entry point from OID provider. Token will not be present immediately. Must start the open id flow
    this.oauthHandler = this.authService.events
        .subscribe((e: OAuthEvent) => {
          console.info("OAuth logout")
          if (e.type === 'logout') {
            this.appAuth.removeAuthorization();
          }
        });

    // IMPORTANT: this reloads the open id flow and tries login, which fires the token_received handled in constructor
    this.userService.initOpenId();
  }

  ngOnDestroy(): void {
    if (this.oauthHandler)
      this.oauthHandler.unsubscribe();
    this.oauthHandler = undefined;
  }
}
