import {Component, Input, OnInit} from '@angular/core';
import {MapMarker, MapWidget} from "../models/map-widget";
import {Community} from "../../../../models/Community";
import {CommunityService} from "../../../../services/community.service";
import {ParametricQuery} from "../../../../models/enea/ParametricQuery";
import {DataService} from "../../../../services/data.service";
import {DatasourceConfiguration} from "../../../Widget";
import {TranslateService} from "@ngx-translate/core";
import {DropdownChangeEvent} from "primeng/dropdown";

@Component({
  selector: 'app-map-widget-configuration',
  templateUrl: './map-widget-configuration.component.html',
  styleUrls: ['./map-widget-configuration.component.css']
})
export class MapWidgetConfigurationComponent implements OnInit {
  @Input() widget?: MapWidget;
  communities: Community[] = [];
  kpis: ParametricQuery[] = [];
  loading: boolean = false;
  defaultCommunityName: string = ""

  constructor(private communityService: CommunityService, private dataService: DataService, private translateService: TranslateService) {
    if (this.widget && !this.widget?.markers)
      this.widget.markers = [];
  }

  ngOnInit(): void {
    this.translateService.get("LABEL.DEFAULT_COMMUNITY_NAME").subscribe({
      next: result => {this.defaultCommunityName = result}
    });
    this.loadCommunities();
    this.loadParametricQueries();
  }

  loadParametricQueries() {
    this.kpis = [];
    this.dataService.getListOfFunctions().subscribe({
      next: result => this.kpis = result,
      error: err => console.error(err)
    });
  }

  private loadCommunities() {
    this.communities = [];
    this.communityService.getCommunities().subscribe({
      next: result => {
        this.communities = result;
        this.communities.unshift(Community.createDefaultUserCommunityEntry(this.defaultCommunityName));
      },
      error: err => console.error(err)
    });
  }

  addCommunity() {
    if (this.widget) {
      if (!this.widget.markers)
        this.widget.markers = [];

      this.widget.markers.push({ kpis: []} as MapMarker);
    }
  }

  removeMarker(marker: MapMarker, i: number) {
    this.widget?.markers?.splice(i, 1);
  }

  addKpi(marker: MapMarker) {
     marker.kpis?.push(new DatasourceConfiguration())
  }

  removeKpi(marker: MapMarker, kpiIndex: number) {
     marker?.kpis?.splice(kpiIndex, 1);
  }
}
